from keras.models import Model, Sequential
from keras.layers.core import Dense, Dropout, Activation
from keras.layers import Embedding, merge, Input
from keras.optimizers import adam
import keras.layers.recurrent
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from keras.utils.np_utils import to_categorical
import numpy as np
import gensim
import random
basepath = "/media/sda3/veritas/Wikipedia/new_experiment/EXP2/Second_Train/"
detail_labels = []
with open(basepath+"0data/TRAINLABEL2nd.txt", "r",encoding='utf8') as f:
    for line in f:
        detail_labels.append(line.rstrip("\n").split("\t")[3])


detaillabels = to_categorical(np.asarray(detail_labels))

epoch_num = 100

###############start
MAX_SEQUENCE_SEN_LENGTH = 150
MAX_SEQUENCE_PARA_LENGTH = 350
MAX_SEQUENCE_CTTL_LENGTH = 100



EMBEDDING_DIM_cttl = 50
EMBEDDING_DIM_sen = 100
EMBEDDING_DIM_para = 180
hidden_dim_cttl = 32#random.choice([18,32,50])
hidden_dim_sen = 64#random.choice([50,64,80])
hidden_dim_para = 128#random.choice([100,128,150])
dim_full_connect = 150#random.choice([128,200,256])
BATCH_SIZE = 1024#random.choice([128,256,512,1024])
lr_num = 0.001#random.choice([0.001,0.0005])
sendata = open(basepath+"train/processed_firstsentence.txt",encoding='utf-8')
paradata = open(basepath+"train/processed_firstpara.txt",encoding='utf-8')
cttldata = open(basepath+"train/processed_section1title.txt",encoding='utf-8')


sen=[]
for line in sendata:
    sen.append(line.lower())


para=[]
for line in paradata:
    para.append(line.lower())


cttl=[]
for line in cttldata:
    cttl.append(line.lower())


tokenizer_sen = Tokenizer()
tokenizer_sen.fit_on_texts(sen)
sequences_sen = tokenizer_sen.texts_to_sequences(sen)
word_index_sen = tokenizer_sen.word_index

tokenizer_para = Tokenizer()
tokenizer_para.fit_on_texts(para)
sequences_para = tokenizer_para.texts_to_sequences(para)
word_index_para = tokenizer_para.word_index

tokenizer_cttl = Tokenizer()
tokenizer_cttl.fit_on_texts(cttl)
sequences_cttl = tokenizer_cttl.texts_to_sequences(cttl)
word_index_cttl = tokenizer_cttl.word_index

data_sen = pad_sequences(sequences_sen, maxlen = MAX_SEQUENCE_SEN_LENGTH)
data_para = pad_sequences(sequences_para, maxlen=MAX_SEQUENCE_PARA_LENGTH)
data_cttl = pad_sequences(sequences_cttl, maxlen=MAX_SEQUENCE_CTTL_LENGTH)
indices = np.arange(detaillabels.shape[0])

np.random.seed(642)
np.random.shuffle(indices)
data_sen = data_sen[indices]
data_para = data_para[indices]
data_cttl = data_cttl[indices]
detaillabels = detaillabels[indices]

model_sen = gensim.models.word2vec.Word2Vec.load(basepath+"data/SEN.model")
num_words_sen = len(word_index_sen) + 1
embedding_matrix_sen = np.zeros((num_words_sen, EMBEDDING_DIM_sen))
for word, i in word_index_sen.items():
    if i >= num_words_sen:
        continue
    if word in model_sen:
        embedding_vector = model_sen[word]
        embedding_matrix_sen[i] = embedding_vector


model_para = gensim.models.word2vec.Word2Vec.load(basepath+"data/PARA.model")
num_words_para = len(word_index_para) + 1
embedding_matrix_para = np.zeros((num_words_para, EMBEDDING_DIM_para))
for word, i in word_index_para.items():
    if i >= num_words_para:
        continue
    if word in model_para:
        embedding_vector = model_para[word]
        embedding_matrix_para[i] = embedding_vector


model_cttl = gensim.models.word2vec.Word2Vec.load(basepath+"data/CTTL.model")
num_words_cttl = len(word_index_cttl) + 1
embedding_matrix_cttl = np.zeros((num_words_cttl, EMBEDDING_DIM_cttl))
for word, i in word_index_cttl.items():
    if i >= num_words_cttl:
        continue
    if word in model_cttl:
        embedding_vector = model_cttl[word]
        embedding_matrix_cttl[i] = embedding_vector


embedding_layer_sen = Embedding(num_words_sen,
                            EMBEDDING_DIM_sen,
                            weights=[embedding_matrix_sen],
                            input_length=MAX_SEQUENCE_SEN_LENGTH,
                            trainable=True)


embedding_layer_para = Embedding(num_words_para,
                            EMBEDDING_DIM_para,
                            weights=[embedding_matrix_para],
                            input_length=MAX_SEQUENCE_PARA_LENGTH,
                            trainable=True)


embedding_layer_cttl = Embedding(num_words_cttl,
                            EMBEDDING_DIM_cttl,
                            weights=[embedding_matrix_cttl],
                            input_length=MAX_SEQUENCE_CTTL_LENGTH,
                            trainable=True)


sen_input = Input(shape=(MAX_SEQUENCE_SEN_LENGTH,), dtype='int32')
embedded_sen = embedding_layer_sen(sen_input)
para_input = Input(shape=(MAX_SEQUENCE_PARA_LENGTH,), dtype='int32')
embedded_para = embedding_layer_para(para_input)
cttl_input = Input(shape=(MAX_SEQUENCE_CTTL_LENGTH,), dtype='int32')
embedded_cttl = embedding_layer_cttl(cttl_input)

sen_ = Dropout(0.5)(embedded_sen)
senx = keras.layers.recurrent.GRU(hidden_dim_sen)(sen_)

para_ = Dropout(0.5)(embedded_para)
parax = keras.layers.recurrent.GRU(hidden_dim_para)(para_)

cttl_ = Dropout(0.5)(embedded_cttl)
cttlx = keras.layers.recurrent.GRU(hidden_dim_cttl)(cttl_)

cat_sen_para = merge.concatenate([senx, parax,cttlx], axis=1)

pred1 = Dense(dim_full_connect)(cat_sen_para)
pred1_ = Dropout(0.5)(pred1)

pred2 = Dense(output_dim=8,activation='softmax')(pred1_)

model0 = Model([sen_input, para_input,cttl_input], pred2)
model0.summary()
Adam = adam(lr=lr_num)

model0.compile(loss='categorical_crossentropy',optimizer=Adam,metrics=['acc'])

print("start")
history = model0.fit([data_sen, data_para,data_cttl], detaillabels,
          batch_size=BATCH_SIZE,
          epochs=epoch_num)


print("END")
rand_num = random.random()
rand_num = str(rand_num)

model0.save(basepath+"model/e"+str(epoch_num)+rand_num+"model.h5")

with open(basepath+"model/e"+str(epoch_num)+rand_num+"trainprocess.txt","w+") as f:
    f.write("hidden_dim_cttl is "+str(hidden_dim_cttl)+"\n")
    f.write("hidden_dim_sen is " + str(hidden_dim_sen) + "\n")
    f.write("hidden_dim_para is " + str(hidden_dim_para) + "\n")
    f.write("dim_full_connect is " + str(dim_full_connect) + "\n")
    f.write("batch size is " + str(BATCH_SIZE) + "\n")
    f.write("lr is " + str(lr_num) + "\n")
    f.write(str(history.history))



############test_predict
sentest = open(basepath+"test/processed_firstsentence.txt",encoding='utf-8')
paratest = open(basepath+"test/processed_firstpara.txt",encoding='utf-8')
cttltest = open(basepath+"test/processed_section1title.txt",encoding='utf-8')


sen=[]
for line in sentest:
    sen.append(line.lower())


para=[]
for line in paratest:
    para.append(line.lower())


cttl=[]
for line in cttltest:
    cttl.append(line.lower())

sentest.close()
paratest.close()
cttltest.close()

sequences_sentest = tokenizer_sen.texts_to_sequences(sen)
sequences_paratest = tokenizer_para.texts_to_sequences(para)
sequences_cttltest = tokenizer_cttl.texts_to_sequences(cttl)

test_sen = pad_sequences(sequences_sentest, maxlen=MAX_SEQUENCE_SEN_LENGTH)
test_sen.shape
test_para = pad_sequences(sequences_paratest, maxlen=MAX_SEQUENCE_PARA_LENGTH)
test_para.shape
test_cttl = pad_sequences(sequences_cttltest, maxlen=MAX_SEQUENCE_CTTL_LENGTH)
test_cttl.shape

predictresult = model0.predict([test_sen, test_para,test_cttl], batch_size=BATCH_SIZE, verbose=1)

'''
outputwrite2 = open(basepath+"final_result/e"+str(epoch_num)+rand_num+"PREDICTresult8.txt", 'w+')
for x in predictresult:
    outputwrite2.write(str(np.argmax(x))+'\n')


outputwrite2.close()
print('END')
'''

outputwrite3 = open(basepath+"final_result/e"+str(epoch_num)+rand_num+"namePREDICTresult8.txt", 'w+',encoding='utf8')

index = 0
with open(basepath+"spiderresult/name.txt","r",encoding='utf8') as f:
    for line in f:
        x = predictresult[index]
        outputwrite3.write(line.rstrip("\n")+"\t"+str(np.argmax(x))+"\n")
        index = index + 1

outputwrite3.close()
print("end")
